#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-dgas-client-config v. 1.1.3
#
# Post-installation script for configuring the gLite DGAS clients
# Guillermo Diez-Andino Sancho <Guillermo.Sancho@cern.ch>
#
# Version info: $Id: glite-dgas-client-config.py,v 1.3.4.12.2.7 2005/12/08 08:47:52 gdiez Exp $
#
# Usage: python glite-dgas-client-config [-c|-v|-h|--start|--stop|--status|--subservice|--help]
#        -c          print configuration
#        --subservice run script as a subservice
#        --configure configure the service
#        --start      start the service
#        --stop      stop the service
#        --status      show the service status
#        -v          print version
#        -h,--help   print usage info
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################
# import system libraries
import os,string,socket
import sys
import getopt
import commands
import time

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams

# Set global variables here 
global params                     # all config values from the XML file

#Configuration files: dgas_atmClient.conf, dgas_ce_pushd.conf and dgas_gianduia.conf

class glite_dgas_client:

    def __init__(self):
        self.verbose = 0
        self.version = "1.1.3"
        self.name = "glite-dgas-client"
        self.friendly_name = "gLite DGAS clients"
        
        self.gianduiaProcesses = list()
        self.gianduiaProcesses.append("/opt/glite/sbin/glite-dgas-ceServerd-had.pl")
        self.gianduiaProcesses.append("/opt/glite/sbin/dgasCeServerd")
        self.gianduiaProcesses.append("/opt/glite/sbin/dgas_gianduia.pl")
        
        self.pushdProcesses = ["/opt/glite/sbin/dgas-ce-pushd.pl"]
            
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf     print the service configuration'
        print '    --subservice        run script as a subservice'
        print '    --configure         configure the service'
        print '    --start             start the service'
        print '    --stop              stop the service'
        print '    --status            shows the service status'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '\n'
        
        
    def backup(self,path):
        backupPath = "%s.1" % path
        if os.path.exists(backupPath):
            os.remove(backupPath)
        if os.path.exists(path):
            os.rename(path,backupPath)

    def start(self):
        error_level = 0
        if service.startGianduia() != 0:
            error_level = 1
        if service.startPushd() != 0:
            error_level = 1    
            
        return error_level
        
    def stop(self):
        error_level = 0
        if service.stopGianduia() != 0:
            error_level = 1
        if service.stopPushd() != 0:
            error_level = 1    
        return error_level
        
    def status(self):
        error_level = 0
        if service.statusGianduia() != 0:
            error_level = 1
        if service.statusPushd() != 0:
            error_level = 1    
        return error_level
        
    def startGianduia(self):
        gianduiaServer = '%s/sbin/glite-wl-dgas-ce-gianduia start' % (os.environ['GLITE_LOCATION'])
        returnCode = 0
        
        self.stopGianduia()
        os.system(gianduiaServer)
        time.sleep(7)
        for process in self.gianduiaProcesses:
            time.sleep(2)
            if glib.getPID(process) == 0:
                glib.printErrorMessage("The %s daemon has not been properly started" %process)
                returnCode = 1
            else:
                print("The %s daemon has been properly started" %process),
                glib.printOkMessage()
                
        return returnCode
        
    def stopGianduia(self):
        returnCode = 0
        for process in self.gianduiaProcesses:
            if glib.getPID(process) != 0:
                glib.stopProcess(process)
                if glib.getPID(process) != 0:
                    glib.printErrorMessage("The process %s has not been properly stopped" %process)
                    returnCode = 1
            else:
                print("The process %s is already stopped" %process),
                glib.printOkMessage()
        return returnCode
    
    def statusGianduia(self):
        error_level = 0
        for process in self.gianduiaProcesses:
            pid = glib.getPID(process)
            if pid != 0:
                print "The process %s is running" %process
            else:
                print "The process %s is not running" %process
                error_level = 1
        return error_level
        
    def startPushd(self):
        pushdServer =  '%s/sbin/glite-dgas-ce-pushd start' % (os.environ['GLITE_LOCATION'])
        returnCode = 0
        
        self.stopPushd()
        os.system(pushdServer)
        for process in self.pushdProcesses:
            if glib.getPID(process) == 0:
                glib.printErrorMessage("The %s daemon has not been properly started" %process)
                returnCode = 1
            else:
                print("The %s daemon has been properly started" %process),
                glib.printOkMessage()
                
        return returnCode
        
    def stopPushd(self):
        returnCode = 0
        for process in self.pushdProcesses:
            if glib.getPID(process) != 0:
                glib.stopProcess(process)
                if glib.getPID(process) != 0:
                    glib.printErrorMessage("The process %s has not been properly stopped" %process)
                    returnCode = 1
            else:
                print("The process %s is already stopped" %process),
                glib.printOkMessage()
        return returnCode
    
    def statusPushd(self):
        error_level = 0
        for process in self.pushdProcesses:
            pid = glib.getPID(process)
            if pid != 0:
                print "The process %s is running" %process
            else:
                print "The process %s is not running" %process
                error_level = 1
        return error_level
    
    def addProcess(self,process):
        self.gianduiaProcesses.append(process)

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):
        service.stop()
        #Creating the configuration file dgas_atmClient.conf
        path = "%s/etc/dgas_atmClient.conf" % os.environ['GLITE_LOCATION']
        self.backup(path)

        file = open(path,'w')
        file.write("res_acct_PA_id = \"%s\"\n" % params['dgas-client.atmClient.resource.PA.id'])
        file.write("res_acct_bank_id = \"%s\"\n" % params['dgas-client.atmClient.resource.Bank.id'])
        file.write("economicAccounting = \"%s\"\n" % params['dgas-client.atmClient.economicAccounting'])
        file.close()
        print "Configuration file %s created" %path,
        glib.printOkMessage()
        
        #Creating the configuration file dgas user
        print "Creating the user %s " %params['dgas-client.ce_pushd.gridUser'],
        glib.add_user(params['dgas-client.ce_pushd.gridUser'])
        glib.printOkMessage()

        
        #Creating the configuration file dgas_ce_pushd.conf
        path = "%s/etc/dgas_ce_pushd.conf" % os.environ['GLITE_LOCATION']
        self.backup(path)
        file = open(path,'w')
        file.write("gridUser = \"%s\"\n" %params['dgas-client.ce_pushd.gridUser'])
        file.write("dgasURDir = \"%s\"\n" % params['dgas-client.ce_pushd.dgasURDir'])
        file.write("dgasErrDir = \"%s\"\n" % params['dgas-client.ce_pushd.dgasErrDir'])
        file.write("qDepth = \"%s\"\n" % params['dgas-client.ce_pushd.qDepth'])
        file.write("qMult = \"%s\"\n" % params['dgas-client.ce_pushd.qMult'])
        file.write("lockFileName = \"%s\"\n" % params['dgas-client.ce_pushd.lockFileName'])
        file.write("logFileName = \"%s\"\n" % params['dgas-client.ce_pushd.logFileName'])
        file.write("mainPollInterval = \"%s\"\n" % params['dgas-client.ce_pushd.mainPollInterval'])
        file.write("queuePollInterval = \"%s\"\n" % params['dgas-client.ce_pushd.queuePollInterval'])
        if(params['dgas-client.ce_pushd.defaultUserHLR'] != ""):
            file.write("defaultUserHLR = \"%s\"\n" % params['dgas-client.ce_pushd.defaultUserHLR'])
        file.write("forceLocalFirst = \"%s\"\n" % params['dgas-client.ce_pushd.forceLocalFirst'])
        file.write("forceLocalOnly = \"%s\"\n" % params['dgas-client.ce_pushd.forceLocalOnly'])
        file.close()
        print "Configuration file %s created" %path,
        glib.printOkMessage()
        
        #Creating the configuration file dgas_gianduia.conf
        path = "%s/etc/dgas_gianduia.conf" % os.environ['GLITE_LOCATION']
        self.backup(path)
        file = open(path, 'w')
        file.write("chocolateBox = \"%s\"\n" % params['dgas-client.gianduia.chocolateBox'])
        file.write("gianduiottiBox = \"%s\"\n" % params['dgas-client.gianduia.gianduiottiBox'])
        file.write("garbageCollector = \"%s\"\n" % params['dgas-client.gianduia.garbageCollector'])
        file.write("lockFileName = \"%s\"\n" % params['dgas-client.gianduia.lockFileName'])
        file.write("logFileName = \"%s\"\n" % params['dgas-client.gianduia.logFileName'])
        file.write("mainPollInterval = \"%s\"\n" % params['dgas-client.gianduia.mainPollInterval'])
        file.write("queuePollInterval = \"%s\"\n" % params['dgas-client.gianduia.queuePollInterval'])
        file.write("lsfAcctLogDir = \"%s\"\n" % params['dgas-client.gianduia.lsfAcctLogDir'])
        file.write("pbsAcctLogDir = \"%s\"\n" % params['dgas-client.gianduia.pbsAcctLogDir'])
        file.write("keyList = \"%s\"\n" % params['dgas-client.gianduia.keyList'])
        file.write("ldifDefaultFiles = \"%s\"\n" % params['dgas-client.gianduia.ldifDefaultFiles'])
        if(params['dgas-client.gianduia.glueLdifFile'] !=""):
            file.write("glueLdifFile = \"%s\"\n" % params['dgas-client.gianduia.glueLdifFile'])
        file.write("dgasCeServerdLock = \"%s\"\n" % params['dgas-client.CeServerd.lockFileName'])
        file.write("dgasCeServerdLog = \"%s\"\n" % params['dgas-client.CeServerd.logFileName'])
        file.write("dgasCeServerdHadLock = \"%s\"\n" % params['dgas-client.CeServerd.hadlockFileName'])
        file.close()        
        print "Configuration file %s created " %path,
        glib.printOkMessage()
        
        #Creating the configuration file dgas_ce_getAcctLogd.conf
        path = "%s/etc/dgas_ce_getAcctLogd.conf" % os.environ['GLITE_LOCATION']
        self.backup(path)
        
        file = open(path, 'w')
        
        if params['use.log.parser']=="true":
            file.write("listenerActive = \"true\"\n")
        else:
            file.write("listenerActive = \"false\"\n")
        file.write("aclFile = \"%s\"\n" % params['dgas-client.getAcctLogd.aclFile'])
        file.write("listeningPort = \"%s\"\n" % params['dgas-client.getAcctLogd.listeningPort'])
        file.write("logFile = \"%s\"\n" % params['dgas-client.getAcctLogd.logFileName'])
        file.write("lockFile = \"%s\"\n" % params['dgas-client.getAcctLogd.lockFileName'])
        file.write("outputFile = \"%s\"\n" % params['dgas-client.getAcctLogd.outputFile'])
        file.write("outputDir = \"%s\"\n" % params['dgas-client.getAcctLogd.outputDir'])
        
        file.close()        
        print "Configuration file %s created " %path,
        glib.printOkMessage()
        
        #Creating the aclFile
        print "Creating the file %s" % params['dgas-client.getAcctLogd.aclFile'],
        if (params['use.log.parser']=="true"):
            self.backup(params['dgas-client.getAcctLogd.aclFile'])
            file = open(params['dgas-client.getAcctLogd.aclFile'],'w')
            file.write("%s\n" %params['log.parser.address'])
            file.close()
        
        return 0
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

    if not os.geteuid() == 0:
        
        params['installer.export.filename'] = glib.checkSetEnvFile(params['installer.export.filename'])
        glib.printInfoMessage("gLite environment file will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
        if os.path.exists(params['GLITE_LOCATION']):
            if not os.access(params['GLITE_LOCATION'], os.W_OK):
                print "    GLITE_LOCATION is set to the non-writable directory %s -" % params['GLITE_LOCATION'],
                params['GLITE_LOCATION'] = "%s" % (installPrefix)
                print "it will be changed to %s" % params['GLITE_LOCATION']
            else:
                if not params['GLITE_LOCATION'] == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if not os.path.exists(params['GLOBUS_LOCATION']):
            print "    GLOBUS_LOCATION is set to the non-existing directory %s " % (params['GLOBUS_LOCATION']),
            params['GLOBUS_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GLOBUS_LOCATION']

        if not os.path.exists(params['GPT_LOCATION']):
            print "    GPT_LOCATION is set to the non-existing directory %s " % (params['GPT_LOCATION']),
            params['GPT_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GPT_LOCATION']

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        print "\n"

    # Standard configuration
    glib.export('GLITE_LOCATION');
    glib.export('GLOBUS_LOCATION')
    glib.export('GLITE_LOCATION_VAR');
    glib.export('GLITE_LOCATION_LOG');
    
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)

    # bin and lib paths
    glib.addEnvPath("PATH","${GLITE_LOCATION}/externals/bin")
    glib.addEnvPath("PATH","${GLITE_LOCATION}/bin")

    glib.addEnvPath("LD_LIBRARY_PATH","${GLOBUS_LOCATION}/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","${GLITE_LOCATION}/lib")

    # Set environment
    glib.setUserEnv()

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # Get an instance of the library class
    params = ConfigParams()

    # Get an instance of the library class
    glib = gLib()
    
    # Load parameters
    if glib.loadConfiguration("%s/../glite-ce.cfg.xml" % glib.getScriptPath(),params):
        print "It was not possible to properly load the glite-ce.cfg.xml file"
        sys.exit(1)
		
    if glib.loadConfiguration("%s/../glite-dgas-client.cfg.xml" % glib.getScriptPath(), params) != 0:
        glib.printErrorMessage("Problems parsing the xml file %s/../glite-dgas-client.cfg.xml" %glib.getScriptPath())
        sys.exit(1)

    #useparser = params['use.log.parser']
    #parserAddress = params['log.parser.address']
    
    if not params.has_key('use.log.parser'):
        params['use.log.parser'] = 'false'
	params['log.parser.address'] = ' '
	
    if params['use.log.parser']=='true':
        if(len(params['log.parser.address'].split('.'))==4): #This is an ip address
            try:
                params['log.parser.address'] = socket.gethostbyaddr(params['log.parser.address'])[0]
            except Exception, ex:
                glib.printErrorMessage("The log.parser.address with value %s can not be resolved to a hostname" %params['log.parser.address']),
                sys.exit(1)				

    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    
    # set verbosity
    if params.has_key('glite.installer.verbose'):
        if (params['glite.installer.verbose'] == "true"):
            verbose=1
        else:
            verbose=0
    else:
        verbose = 0
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
    service = glite_dgas_client()    
    
    path = '%s/etc' % os.environ['GLITE_LOCATION']
    if not os.path.exists(path):
        glib.printFailedMessage("%s/etc does not exist! Please verify your gLite installation" % os.environ['GLITE_LOCATION'])
        sys.exit (1)
    else:
        print '%s/etc exists' % os.environ['GLITE_LOCATION'],
        glib.printOkMessage()

    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','start','stop','status','configure','subservice','siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "[FAILED] Unknown options(s)")
        sys.exit(1)
        
    if (params['use.log.parser']=="true"):
        service.addProcess("/opt/glite/sbin/dgas-ce-getAcctLogd")
    
    # Check cli options
    configure = 0
    subservice = 0
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
        if o == "--start":
            if service.start() == 0:
                print "The %s was successfully started" % service.friendly_name,
                glib.printOkMessage()
                sys.exit(0)
            else:
                print "An error occurred while starting the %s service" % service.friendly_name,
                glib.printFailedMessage()
                sys.exit(1)
        if o == "--stop":
            if service.stop() == 0:
                print "The %s was successfully stopped" % service.friendly_name,
                glib.printOkMessage()
                sys.exit(0)
            else:
                print "An error occurred while stopping the %s service" % service.friendly_name,
                glib.printFailedMessage()
                sys.exit(1)
        if o == "--status":
            sys.exit(service.status())
        if o == "--subservice":
            subservice = 1    
        if o == "--configure":
            configure = 1
            
    if configure == 1:
        if subservice == 0:
            service.copyright()
            service.showVersion()
            
        if service.configure() == 0:
            print "%s configuration successfully completed " % service.friendly_name,
            glib.printOkMessage()
            glib.registerService()
            sys.exit(0)
        else:
            print "An error occurred while configuring the %s service" % service.friendly_name,
            glib.printFailedMessage()
            sys.exit(1)
    else:        
        # Print configuration parameters
        if verbose:
            glib.print_params(params)
            service.copyright()
            service.showVersion()
            service.banner()
            service.usage()    
            sys.exit(0)
